/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.executePlan.dto.planBatch;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
* Restful接口--资源详情--数据传输对象：计划执行批次表
*
* createdate:  2022-12-06 11:32:04  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@JsonSerialize
@ApiModel(description = "计划执行批次表")
public class AtuPlanBatchPageDto implements Serializable
{

    /**
     * 主键<br>
     */
    @ApiModelProperty("主键")
    @Size(max = 32, message="主键-id: 数据长度不能 > 32" )
    private String id;
    /**
     * 计划主键<br>
     */
    @ApiModelProperty("计划主键")
    @Size(max = 32, message="计划主键-planId: 数据长度不能 > 32" )
    private String planId;
    /**
     * 任务批次<br>
     */
    @ApiModelProperty("任务批次")
    private Integer batch;
    /**
     * 状态;0-等待中，1-执行中，2-已完成，3-已取消<br>
     */
    @ApiModelProperty("状态;0-等待中，1-执行中，2-已完成，3-已取消")
    @Size(max = 1, message="状态;0-等待中，1-执行中，2-已完成，3-已取消-status: 数据长度不能 > 1" )
    private String status;
    /**
     * 调度开始时间<br>
     */
    @ApiModelProperty("调度开始时间")
    private Date startTime;
    /**
     * 调度结束时间<br>
     */
    @ApiModelProperty("调度结束时间")
    private Date endTime;
    /**
     * 任务总数<br>
     */
    @ApiModelProperty("任务总数")
    private Integer taskTotal;
    /**
     * 成功任务数<br>
     */
    @ApiModelProperty("成功任务数")
    private Integer successNum;
    /**
     * 执行失败数<br>
     */
    @ApiModelProperty("执行失败数")
    private Integer execFailNum;
    /**
     * 断言失败数<br>
     */
    @ApiModelProperty("断言失败数")
    private Integer assertFailNum;
    /**
     * 超时任务数<br>
     */
    @ApiModelProperty("超时任务数")
    private Integer timeoutNum;
    /**
     * 取消任务数<br>
     */
    @ApiModelProperty("取消任务数")
    private Integer cancelNum;
    /**
     * 成功率<br>
     */
    @ApiModelProperty("成功率")
    @Size(max = 10, message="成功率-successRate: 数据长度不能 > 10" )
    private String successRate;
    /**
     * 创建时间<br>
     */
    @ApiModelProperty("创建时间")
    private Date createdTime;

    @ApiModelProperty("计划名称")
    private String planName;

    @ApiModelProperty("录屏设置;0-不保存，1-全部保存，2-出错保存")
    @Size(max = 1, message="录屏设置;0-不保存，1-全部保存，2-出错保存-screenRecordSet: 数据长度不能 > 1" )
    private String screenRecordSet;

    /**
     * 脚本总数<br>
     */
    @ApiModelProperty("脚本总数")
    private Integer scriptNum;

    /**
     * 执行人<br>
     */
    @ApiModelProperty("执行人")
    @Size(max = 32, message="执行人-executedBy: 数据长度不能 > 32" )
    private String executedBy;
    /**
     * 触发类型；1-自动，2-手动，3-devops<br>
     */
    @ApiModelProperty("触发类型；1-自动，2-手动，3-devops")
    @Size(max = 1, message="触发类型；1-自动，2-手动，3-devops-triggerType: 数据长度不能 > 1" )
    private String triggerType;

    @ApiModelProperty("耗时")
    private String elapsedTime;

    public String getElapsedTime() {
        return elapsedTime;
    }

    public void setElapsedTime(String elapsedTime) {
        this.elapsedTime = elapsedTime;
    }

    public String getScreenRecordSet() {
        return screenRecordSet;
    }

    public void setScreenRecordSet(String screenRecordSet) {
        this.screenRecordSet = screenRecordSet;
    }

    public String getPlanName() {
        return planName;
    }

    public void setPlanName(String planName) {
        this.planName = planName;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getPlanId() {
        return planId;
    }

    public void setPlanId(String planId) {
        this.planId = planId;
    }

    public Integer getBatch() {
        return batch;
    }

    public void setBatch(Integer batch) {
        this.batch = batch;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Integer getTaskTotal() {
        return taskTotal;
    }

    public void setTaskTotal(Integer taskTotal) {
        this.taskTotal = taskTotal;
    }

    public Integer getSuccessNum() {
        return successNum;
    }

    public void setSuccessNum(Integer successNum) {
        this.successNum = successNum;
    }

    public Integer getExecFailNum() {
        return execFailNum;
    }

    public void setExecFailNum(Integer execFailNum) {
        this.execFailNum = execFailNum;
    }

    public Integer getAssertFailNum() {
        return assertFailNum;
    }

    public void setAssertFailNum(Integer assertFailNum) {
        this.assertFailNum = assertFailNum;
    }

    public Integer getTimeoutNum() {
        return timeoutNum;
    }

    public void setTimeoutNum(Integer timeoutNum) {
        this.timeoutNum = timeoutNum;
    }

    public Integer getCancelNum() {
        return cancelNum;
    }

    public void setCancelNum(Integer cancelNum) {
        this.cancelNum = cancelNum;
    }

    public String getSuccessRate() {
        return successRate;
    }

    public void setSuccessRate(String successRate) {
        this.successRate = successRate;
    }

    public Date getCreatedTime() {
        return createdTime;
    }

    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    public Integer getScriptNum() {
        return scriptNum;
    }

    public void setScriptNum(Integer scriptNum) {
        this.scriptNum = scriptNum;
    }

    public String getExecutedBy() {
        return executedBy;
    }

    public void setExecutedBy(String executedBy) {
        this.executedBy = executedBy;
    }

    public String getTriggerType() {
        return triggerType;
    }

    public void setTriggerType(String triggerType) {
        this.triggerType = triggerType;
    }

    // TODO 请完成资源详情展示的属性
}