/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2023 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.inputSet.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.db.entity.AtuInputSet;
import net.northking.cctp.scriptcase.dto.inputSet.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 输入项数据集Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AtuInputSetApiService extends ExcelService
{
  /**
   * 查询输入项数据集列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<AtuInputSetDetailDto> pagingQuery(QueryByPage<AtuInputSetQueryDto> queryByPage);

  /**
   * 新增输入项数据集
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(AtuInputSetAddDto dto);

  /**
   * 更新输入项数据集
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AtuInputSetUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取输入项数据集
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  AtuInputSetDetailDto findByPK(String uuid);

  /**
   * 删除输入项数据集
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<AtuInputSetAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<AtuInputSetUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  List<InputSet> saveInputSet(AtuInputSetSaveDto dto);

    void downloadInputTemplate(HttpServletResponse response, HttpServletRequest request,List<String> firstLine);

    void downloadInputData(HttpServletResponse response, HttpServletRequest request,AtuInputSetDownloadDto dto);

    String importSetExcel(MultipartFile file, AtuInputSetFileDto dto);

  InputSetResultDto queryInputData(String scriptId,Boolean flag);

  /**
   * 根据环境和脚本查询脚本输入项信息
   * @param queryDto 查询参数 环境id集合+脚本id集合
   * @return 查询结果
   */
    List<AtuInputSet> queryInputByEnvIdAndScriptId(AtuPlanInputQueryDto queryDto);

  InputSetResultDto queryInputSetByCase(AtuInputSetCaseDto dto);

    List<AtuInputSet> queryInputByEnvIdAndScriptIdMap(Map<String, List<String>> envScriptMap);

    Boolean checkDataSetDelete(AtuInnputDataSetQueryDto queryDto);

  List<AtuInputDataSetResultDto> selectLinkBySetId(List<String> setList);
}
