/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2023 . All rights reserved.
*
*/
package net.northking.cctp.project.api.env.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.project.dto.env.*;

import java.util.List;

/**
 * 系统环境被测系统信息Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CpProjectEnvSysApiService extends ExcelService
{
  /**
   * 查询系统环境被测系统信息列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CpProjectEnvSysDetailDto> pagingQuery(QueryByPage<CpProjectEnvSysQueryDto> queryByPage);

  /**
   * 新增系统环境被测系统信息
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(CpProjectEnvSysAddDto dto);

  /**
   * 更新系统环境被测系统信息
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CpProjectEnvSysUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取系统环境被测系统信息
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  CpProjectEnvSysDetailDto findByPK(String uuid);

  /**
   * 删除系统环境被测系统信息
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<CpProjectEnvSysAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<CpProjectEnvSysUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  /**
   * 查询系统环境下被测系统下拉选项
   * @param projectId 系统id
   * @param envId 环境ID
   * @return 结果集
   */
    List<CpProjectEnvSysSelectDto> findSelect(String projectId, String envId);

  /**
   * 查询被测系统地址
   * @param searchDto 查询参数
   * @return 系统访问地址
   */
    String querySysUrl(CpProjectEnvSysUrlSearchDto searchDto);

  /**
   * 同步系统环境被测系统信息
   * @param projectId 系统id
   * @param envIds 环境id集合
   */
    void syncProjectEnvSys(String projectId, List<String> envIds);
}
