/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.dw.api.service.MtCaseDesignChartApiService;
import net.northking.cctp.dw.dto.MtCaseDesignChartDetailDto;
import net.northking.cctp.dw.dto.MtCaseDesignChartQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

@Api(tags = "用例设计统计图表 ")
@RequestMapping("/v1")
@RestController
public class MtCaseDesignChartQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(MtCaseDesignChartQueryCtrl.class);

  @Autowired
  private MtCaseDesignChartApiService apiService;

  /**
   * 查询用户列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：用例设计统计图表 ")
  @PostMapping(value = "/paging/query/MtCaseDesignCharts",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<MtCaseDesignChartDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<MtCaseDesignChartQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<MtCaseDesignChartDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<MtCaseDesignChartDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

  @ApiOperation(value = "导出Excel：用例设计统计图表 ")
  @PostMapping(value = "/excel/export/MtCaseDesignCharts",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void exportExcel(
      @RequestBody @Validated QueryByPage<MtCaseDesignChartQueryDto> queryByPage,
      HttpServletResponse response)
  {
    // 定义导出的Excel在客户端浏览器上提示下载的文件名，不写后缀
    String fileName = "CCTP-用例设计统计图表 ";
    apiService.exportExcel(response, queryByPage, fileName);
  }

  @ApiOperation(value = "下载Excel模板：用例设计统计图表 ")
  @PostMapping(value = "/excel/template/MtCaseDesignCharts",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void downloadTemplate(HttpServletResponse response)
  {
    String fileName = "CCTP模板-用例设计统计图表 ";
    apiService.downloadTemplate(response, fileName);
  }

  @ApiOperation(value = "导入Excel：用例设计统计图表 ")
  @PostMapping(value = "/excel/import/MtCaseDesignCharts",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  public ResultWrapper<Integer> importExcel(
      @RequestParam("file") MultipartFile file)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    int count = apiService.importExcel(file);

    return wrapper.success(count);
  }

}


